Attribute VB_Name = "EN_Cartography"
'########################################################
'#      Cartography module                              #
'########################################################
'#  Author: SCHRICKE Anthony                            #
'#  Version: 1.1b                                       #
'#  Date: 15/12/2009                                    #
'#  Creation date: 20/08/2009                           #
'#  Login (developpez.com) : Tonioyo                    #
'#  Description: This Tool for developpers list the     #
'#      calls procedures (Subs and functions).          #
'#      This module is available for all Office         #
'#      applications.                                   #
'#      To be tested for Visual Basic application.      #
'#  Updates:                                            #
'#      v 1.0 : First version.                          #
'#      v 1.1 : Func. added - Line analysis.            #
'########################################################


'global variant table for the structure of the code
Dim CodeStruct() As Variant

'********************************************************
'*      CartographyCode                                 *
'********************************************************
'* Description:                                         *
'*      The main sub to cartography your code.          *
'*                                                      *
'* Parameters:                                          *
'*      StartProc (String) : The start function targeted*
'*                                                      *
'********************************************************

Public Sub CartographyCode(ByRef StartProc As String)
  
  'Reformat the string parameter if parenteses are presents
  If InStr(StartProc, "(") > 0 Then StartProc = Left(StartProc, InStr(StartProc, "(") - 1)
  LoadCodeStruct 'loading of the code structure
  ReadCallStack SearchStartProcedure(StartProc) 'research the calls starting by the method passed in parameter
  
End Sub

'********************************************************
'*      LoadCodeStruct                                  *
'********************************************************
'* Description:                                         *
'*      This is here where all functions and subs are   *
'*    loaded in memory.                                 *
'*                                                      *
'********************************************************

Public Sub LoadCodeStruct()
On Error GoTo Err_LoadCodeStruct

  Dim Comp As VBComponent       'Variant containing a Visual Basic Component
  Dim i, Iter As Long           'Iterators
  Dim CurProc As String         'String containing the name of the current method
  
  Iter = 0                              'Initialization of the iterator
  ReDim Preserve CodeStruct(Iter)       'dynamic reallocation
  
  For Each Comp In Application.VBE.ActiveVBProject.VBComponents       'For each VB component
    For i = 1 To Comp.CodeModule.CountOfLines                         'Brownse each line
      If CurProc <> Comp.CodeModule.ProcOfLine(i, vbext_pk_Proc) Then 'If current metyhod is different from one present on the viewed line
        CurProc = Comp.CodeModule.ProcOfLine(i, vbext_pk_Proc)        'Saving the method name found that become the current method
        If CurProc <> "" Then                                         'If the current method is not an empty string
          ReDim Preserve CodeStruct(Iter)                             'dynamic reallocation in order to save information
                                                                              'Save informations about the method detected
                                                                              'Items saved :
                                                                              '       0 - Component name (module name, worksheet name, ...)
                                                                              '       1 - Method name
                                                                              '       2 - Line number of the beginning of the method
                                                                              '       3 - Number of lines from the method
                                                                              '       4 - Type of the method (getter, setter, function, sub)
          CodeStruct(Iter) = Array(Comp.Name, CurProc, Comp.CodeModule.ProcStartLine(CurProc, vbext_pk_Proc), Comp.CodeModule.ProcCountLines(CurProc, vbext_pk_Proc), vbext_pk_Proc)
          Iter = Iter + 1                                                     'Iterator incrementing
        End If
      End If
    Next
  Next

  
  SortStrArray CodeStruct()     'Sort the array
  Exit Sub
  
Err_LoadCodeStruct:                                             'Error management
  Iter = LoadCodeStructProperty(Comp, CurProc, Iter) - 1        'If an error is thrown it is because the method
                                                                'is a property (getter/setter) then it must search the type of the property
                                                                'and updates the iterator
  Resume Next                                                   'Returning into the loop of the module
End Sub


'********************************************************
'*      LoadCodeStructProperty                          *
'********************************************************
'* Description:                                         *
'*      This is here where all properties are loaded in *
'*   memory.                                            *
'*                                                      *
'* Parameters:                                          *
'*      Comp (VBComponent) : Current component.         *
'*      CurProc (String): Property to be search for.    *
'*      Iter (Long): Iterator of the table in memory.   *
'*                                                      *
'* Return:                                              *
'*      (Long): The iterator updated.                   *
'********************************************************


Public Function LoadCodeStructProperty(ByRef Comp As VBComponent, ByRef CurProc As String, ByRef Iter As Long) As Long
On Error GoTo Err_LoadCodeStructProperty

  Dim ProcKinds(2) As Long          'Array containing all the types of propoerties
  Dim i As Byte                     'Iterator
  
  'Initialization of the array
  ProcKinds(0) = vbext_pk_Get       'Getter
  ProcKinds(1) = vbext_pk_Let       'Getter
  ProcKinds(2) = vbext_pk_Set       'Setter

  'Search the type of the property
  For i = 0 To 2                    'Once found, the values are saved into the structure array, the detection is made on the error thrown.
    CodeStruct(Iter) = Array(Comp.Name, CurProc, Comp.CodeModule.ProcStartLine(CurProc, ProcKinds(i)), Comp.CodeModule.ProcCountLines(CurProc, ProcKinds(i)), ProcKinds(i))
    Iter = Iter + 1                 'Iterator incrementation
    ReDim Preserve CodeStruct(Iter) 'dynamic reallocation
  Next
  
  LoadCodeStructProperty = Iter
  Exit Function

Err_LoadCodeStructProperty:     'Error management
  Iter = Iter - 1               'If an error is thrown it is not the good type then iterator decrementation
  Resume Next                   'and returning into the loop
End Function


'********************************************************
'*      SearchStartProcedure                            *
'********************************************************
'* Description:                                         *
'*      Search the first procedure in memory table in   *
'*    order to know where to start.                     *
'*                                                      *
'* Parameters:                                          *
'*      SearchProc (String) : Procedure to be reseached.*
'*                                                      *
'********************************************************

Public Function SearchStartProcedure(ByRef SearchProc As String) As Long

  Dim Iter As Long                             'Iterator
  
  For Iter = 1 To UBound(CodeStruct())         'Search into the array structure the starting method
    If CodeStruct(Iter)(1) = SearchProc Then   'If it is found
      SearchStartProcedure = Iter              'iterator is saved (It will be used as a pointer)
      Exit For                                 'Break the loop (the next iterations are unusefull)
    End If
  Next
  
End Function

'********************************************************
'*      ReadCallStack                                   *
'********************************************************
'* Description:                                         *
'*      Recursive function what browse all calls.       *
'*                                                      *
'* Parameters:                                          *
'*      SrchItemIter (Long) : Iterator which define the *
'*   searched item.                                     *
'*      Deep (Long - Optional) : The deep of recursive  *
'*   calls.                                             *
'*                                                      *
'********************************************************

Public Sub ReadCallStack(ByRef SrchItemIter As Long, Optional ByRef Deep As Long = 0)
  Dim Iter As Long                      'Iterator
  Dim i As Long                         'Iterator
  Dim StrDeclaration As String          'Line containning the method declaration
  Dim StrProc As String                 'Name of the method
  Dim CM As CodeModule                  'Code module for searching information into
  
  'Searching information into the method pointed by the parameter SrchItemIter.
  For i = CLng(CodeStruct(SrchItemIter)(2)) + 1 To CLng(CodeStruct(SrchItemIter)(2)) + CLng(CodeStruct(SrchItemIter)(3)) - 2
    For Iter = 0 To UBound(CodeStruct())                                                          'running into the structure array
      If CodeStruct(Iter)(1) <> CodeStruct(SrchItemIter)(1) Then                                  'If the name of the method searched is different than the one currently searched (by iteration into the structure array)
        With Application.VBE.ActiveVBProject.VBComponents(CodeStruct(SrchItemIter)(0)).CodeModule 'Using the module into one where searching the calls (pointer passer in parameter)
          If .Find(CodeStruct(Iter)(1), i, 1, i, 255, True, True) Then                            'Searching into the table what listing all custom methods if the pointed method is in the line
            analyseLine Trim(.Lines(i, 1)), SrchItemIter, i, Deep   'Analyse the code line
            ReadCallStack Iter, Deep + 1   'Recursive call (in order to analyse the container of the detected call)
            i = i + 1                      'go to the next line
          End If
        End With
      End If
    Next
  Next
  
End Sub

'********************************************************
'*      ReadProcType                                    *
'********************************************************
'* Description:                                         *
'*      Determine the type of the propertie/procedure.  *
'*                                                      *
'* Parameters:                                          *
'*      Iter (Long) : Iterator to find the type         *
'*                                                      *
'* Returns:                                             *
'*      (String) : The type of the procedure/property   *
'********************************************************

Public Function ReadProcType(ByRef Iter As Long) As String

  Select Case CodeStruct(Iter)(4)  'Pointing the iterator value
    Case vbext_pk_Proc
      ReadProcType = "Proc"        'If it is a procedure return "Proc"
    Case vbext_pk_Get
      ReadProcType = "Get"         'If it is a getter return "Get"
    Case vbext_pk_Let
      ReadProcType = "Let"         'If it is a getter return "Let"
    Case vbext_pk_Set
      ReadProcType = "Set"         'If it is a setter return "Set"
  End Select

End Function


'********************************************************
'*      SortStrArray                                    *
'********************************************************
'* Description:                                         *
'*      Sort an array with string in order to do a      *
'*   dichotomic research later. (for a new version of   *
'*   this tool).                                        *
'*      For now, it sort on the first character only.   *
'*                                                      *
'* Parameters:                                          *
'*      Arr() (Variant) : Array to be sorted            *
'*                                                      *
'********************************************************

'Attention tableau a 2 dimentions
'WARNING - it is for table with 2 dimentions

Public Sub SortStrArray(ByRef Arr() As Variant)
  Dim LenArray As Long                  'Lenght of the table
  Dim Iterator As Long                  'Iterator
  Dim SaveIterator As Long              'Iterator saving
  Dim StockValue As Variant             'Stock a value / an object (in order to do the exchange)
  
  LenArray = UBound(Arr())              'Lenght calculation
  
  If LenArray >= 2 Then                 'If the array have more than one eelement (if there is one element it is not usefull to sort it)
    Iterator = 1                        'Iterator initialization  (Starting at 1 because there is at least one element)
    While Iterator <= LenArray          'While iterator didn't overflow the array
      While Asc(UCase(Arr(Iterator - 1)(1))) > Asc(UCase(Arr(Iterator)(1))) 'While the value / object located before the iterator is greater than the value / object where the iterator is pointing
        'Moving values / objtects
        StockValue = Arr(Iterator - 1)                'Stocking the value / object
        Arr(Iterator - 1) = Arr(Iterator)             'Updating the position of the second value / object
        Arr(Iterator) = StockValue                    'Writting of the stocked value / object at the new location
        If Iterator > 1 Then Iterator = Iterator - 1  'Restart the sort on the first position exchanged
      Wend
      Iterator = Iterator + 1     'Iterator incrementation
    Wend
  End If
End Sub


'********************************************************
'*      DisplayMessage                                  *
'********************************************************
'* Description:                                         *
'*      Display a message from the method detected.     *
'*                                                      *
'* Parameters:                                          *
'*      StrDeclare(String): Declaration of the method.  *
'*      Iterator (Long): Iterator.                      *
'*      LineNumber(Long): Number of the line detected.  *
'*      Deep(Long): Deep for the display.               *
'*      SrchItemIter(Long): Iterator of item searched.  *
'*                                                      *
'********************************************************

Private Sub DisplayMessage(ByRef StrDeclare As String, ByRef Iterator As Long, ByRef LineNumber As Long, ByRef Deep As Long, ByRef SrchItemIter As Long)
        Dim StrProc As String
        
        'Processing the saved string (In order to print a message)
        StrProc = Right(StrDeclare, Len(StrDeclare) - InStr(StrDeclare, CodeStruct(Iterator)(1)) + 1)
        
        'processing the string for display
        If InStrRev(StrProc, ")") > 0 Then StrProc = Left(StrProc, InStrRev(StrProc, ")"))
        
        'Display the call detected into the debug window
        Debug.Print Space(2 * Deep) & CStr(CodeStruct(SrchItemIter)(1)) & " [" & CodeStruct(SrchItemIter)(0) & "(" & LineNumber & ") -> " & CodeStruct(Iterator)(0) & "." & CodeStruct(Iterator)(1) & "{" & ReadProcType(Iterator) & "}] - " & StrProc
        
End Sub

'********************************************************
'*      analyseLine                                     *
'********************************************************
'* Description:                                         *
'*      Analyse the code line providing from a module.  *
'*                                                      *
'* Parameters:                                          *
'*      StrIn(String): Line to be analysed.             *
'*      Iterator (Long): Iterator.                      *
'*      LineNumber(Long): Number of the line.           *
'*      Deep(Long): Deep of the display.                *
'*                                                      *
'********************************************************

Private Sub analyseLine(ByRef StrIn As String, ByRef Iterator As Long, ByRef LineNumber As Long, ByRef Deep As Long)
        Dim ParIndexS As Integer  'Index of start of bloc parenthesis
        Dim ParIndexE As Integer  'Index of end of bloc parenthesis
        Dim SpaIndex As Integer   'Index of space separator
        Dim i As Long             'Iterator
        Dim StrWord As String     'Stock of a word


        If Len(StrIn) > 0 Then                      'If the ligne containnig caracter
          If Left(StrIn, 1) = "'" Then Exit Sub     'If it is a comment do nothing
          ParIndexS = InStr(StrIn, "(")             'Begining first bloc of parenteses affectation
          SpaIndex = InStr(StrIn, Space(1))         'Index first space detected into the string affectation
          
          If SpaIndex = 0 Then SpaIndex = 32767     'If index value is 0, the space caracter is not present. We forces the index to keep coherence into the program
          If ParIndexS = 0 Then ParIndexS = 32767   'If index value is 0, the parenthesis caracter is not present. We forces the index to keep coherence into the program
          
          If SpaIndex = 32767 And ParIndexS = 32767 Then
          'if indexes of begining of parenteses and space values are at 0, it means that there are no more separators on the line
          '  Debug.Print "Mot " & StrIn: Stop
            For i = 0 To UBound(CodeStruct())               'Running into the structure table
              If CodeStruct(i)(1) = StrIn Then              'If the word is found into the structure table
                DisplayMessage StrIn, i, LineNumber, Deep, Iterator   'display of call detected
                Exit For                                    'Unusefull to run the rest of the table
              End If
            Next
            Exit Sub
          End If

          If SpaIndex < ParIndexS Then                      'If a space caracter is present before a parenthese
            StrWord = Trim(Left(StrIn, SpaIndex))           'Word isolation
            'Debug.Print "Word: " & StrWord & vbCrLf & "In the string: " & StrIn: Stop
            analyseLine Trim(Right(StrIn, Len(StrIn) - SpaIndex)), Iterator, LineNumber, Deep  'Analysis of all what is present after the parenthese (rest of the string)
            
            For i = 0 To UBound(CodeStruct())               'Structure table running
              If CodeStruct(i)(1) = StrWord Then            'If the word is found into the structure table
                DisplayMessage StrIn, i, LineNumber, Deep, Iterator   'Display of the call detected
                Exit For                                    'Unusefull to run the rest of the table
              End If
            Next
          Else                                              'If a parenthese is present before a space caracter
            'Parenthesis management
            Dim stockStrCur As Long                         'Index cursor
            Dim stockStrStart As Long                       'Index parenthese openning
            Dim stockStrEnd As Long                         'Index parenthse closing in the bloc viewed
            Dim cpt As Integer                              'Counter
            
            stockStrCur = 0                                 'Initializations
            stockStrEnd = 0
            cpt = 1
            
            'Debug.Print "Parenthese " & StrIn: Stop
            
            stockStrStart = InStr(stockStrCur + 1, StrIn, "(") 'parenthese openning detection
                  
            'detects the number of newer blocks until the first closing parenthese
            Dim c As String * 1       'String caracter to be analysed
            Dim chaine As String      'String operations results
            Dim Detected As Boolean   'To detect the end of the block
            
            'Remaining the right part of the string in order to isolate the parenthesis block detected
            chaine = Right(StrIn, Len(StrIn) - InStr(StrIn, "("))
                                  
            For i = 1 To Len(chaine)  'Running the string
              c = Mid(chaine, i, 1)   'Read the caracter
              Select Case c           'Analyze the caracter
                Case "(":             'If newer block increments the counter blocks (case where there is a block in a block => (xxx (yyy, nnn)) for example)
                  cpt = cpt + 1     'Counter incrementation
                Case ")":             'If a end of block is detected decrementation of the counter
                  cpt = cpt - 1     'Counter decrementation
              End Select
              If cpt = 0 Then       'If the counter value is at 0, the end of block that we wants isolate is detected (case where there will have more that one adjacent parenthesis block => (xxx), (yyy) for example)
                Detected = True     'Put the detection of the end of block to true
                Exit For            'Breaking the loop => Unusefull to run the rest of the line
              End If
            Next i
                                  
            
            
            If Detected Then analyseLine Left(chaine, i - 1), Iterator, LineNumber, Deep 'si la fin de bloc est dtecte, dclenchement de l'analyse de son contenu
            StrWord = Trim(Left(StrIn, stockStrStart - 1))  'isolation du mot qui a appel le bloc de parentheses
            'Debug.Print "Mot par:" & StrWord

            For i = 0 To UBound(CodeStruct())             'Running the structure table
              If CodeStruct(i)(1) = StrWord Then          'If the word if found as method
                DisplayMessage StrIn, i, LineNumber, Deep, Iterator 'Displaying the message of the call
                Exit For                                  'Breaking the loop
              End If
            Next
          End If
        End If
End Sub

