Attribute VB_Name = "FR_Cartography"
'########################################################
'#      Module Cartography                              #
'########################################################
'#  Auteur: SCHRICKE Anthony                            #
'#  Version: 1.1b                                       #
'#  Date: 15/12/2009                                    #
'#  Date de cation: 20/08/2009                         #
'#  Login (developpez.com) : Tonioyo                    #
'#  Description: Cet outils pour developpeurs liste les #
'#      appels de procedures (Sub et function).         #
'#      Ce module est disponible pour toutes les        #
'#      applications offices.                           #
'#      A tester pour l'application Visual Basic.       #
'#  Mises  jour:                                       #
'#      v 1.0: Premire version.                        #
'#      v 1.1: Fonc. ajoute - Analyse de ligne.        #
'########################################################


'Variables globales tableau pour la structure du code
Dim CodeStruct() As Variant

'********************************************************
'*      CartographyCode                                 *
'********************************************************
'* Description:                                         *
'*      Procedure principale pour cartographier le code.*
'*                                                      *
'* Parameters:                                          *
'*      StartProc (String) : La premiere fonction cible*
'*                                                      *
'********************************************************

Public Sub CartographyCode(ByRef StartProc As String)

  
  'Reformate la chaine de caracteres passe en parametres si des parentheses sont prsentes
  If InStr(StartProc, "(") > 0 Then StartProc = Left(StartProc, InStr(StartProc, "(") - 1)
  LoadCodeStruct 'Chargement de la strucutre du code
  ReadCallStack SearchStartProcedure(StartProc) 'recherche des appels  partir de la procedure de dpart
  
End Sub

'********************************************************
'*      LoadCodeStruct                                  *
'********************************************************
'* Description:                                         *
'*      C'est ici que toutes les fonctions et procedures*
'*    sont charges en mmoire.                         *
'*                                                      *
'********************************************************

Public Sub LoadCodeStruct()
On Error GoTo Err_LoadCodeStruct

  Dim Comp As VBComponent       'Variable qui contient un composant VB
  Dim i, Iter As Long           'Itrateurs
  Dim CurProc As String         'Chaine contenant le nom de la procedure courrante
  
  Iter = 0                              'Initialisation de l'iterateur
  ReDim Preserve CodeStruct(Iter)       'Reallocation dynamique du tableau de structure
  
  For Each Comp In Application.VBE.ActiveVBProject.VBComponents        'Pour chaque composant VB
    For i = 1 To Comp.CodeModule.CountOfLines                          'On parcours chaque ligne
      If CurProc <> Comp.CodeModule.ProcOfLine(i, vbext_pk_Proc) Then  'Si la procedure courrante est diffrente de celle prsente sur la ligne observe
                'Break  poser pour vrifier la ligne 65
        CurProc = Comp.CodeModule.ProcOfLine(i, vbext_pk_Proc)         'On sauvegarde le nom de la procdure trouve qui deviens procedure courante.
        If CurProc <> "" Then                                          'Si la procdure courrante n'est pas une chaine vide (Ce qui peux arriver lors de la dtection de la toute premiere procdure)
          ReDim Preserve CodeStruct(Iter)                              'Reallocation dynamique du tableau de structure afin de pouvoir enregistrer les informations
                                                                                     'Sauvegarde des informations de la mthode dtecte.
                                                                                     'Items sauvegardes :
                                                                                     '               0 - Le nom du composant (nom du module de code, feuille
                                                                                     '               1 - Le nom de la procedure
                                                                                     '               2 - Le numero de la ligne de dpart de la procedure
                                                                                     '               3 - Le nombre de lignes de la procedure
                                                                                     '               4 - Le type de la procedure (getter, setter, methode, procedure)
          CodeStruct(Iter) = Array(Comp.Name, CurProc, Comp.CodeModule.ProcStartLine(CurProc, vbext_pk_Proc), Comp.CodeModule.ProcCountLines(CurProc, vbext_pk_Proc), vbext_pk_Proc)
          Iter = Iter + 1                                                            'Incrmentation de l'iterateur
        End If
      End If
    Next
  Next

  
  SortStrArray CodeStruct()     'Trie le tableau
  Exit Sub
  
Err_LoadCodeStruct:                                       'Gestion d'erreur
  Iter = LoadCodeStructProperty(Comp, CurProc, Iter) - 1  ' Si une erreur est produite c'est  cause du fait que la methode est
                                                          ' une proprit (getter/setter) alors on cherche le type de la proprit
                                                          ' et on met  jour l'iterateur.
  Resume Next                                             'On retourne dans la boucle du module
End Sub

'********************************************************
'*      LoadCodeStructProperty                          *
'********************************************************
'* Description:                                         *
'*      C'est ici que les proprits sont               *
'*    charges en mmoire.                              *
'*                                                      *
'* Parameters:                                          *
'*      Comp (VBComponent) : Composant courrant.        *
'*      CurProc (String): Nom de la propit recherche.*
'*      Iter (Long): Iterateur du tableau en mmoire.   *
'*                                                      *
'* Return:                                              *
'*      (Long): L'iterateur mis  jour.                 *
'********************************************************


Public Function LoadCodeStructProperty(ByRef Comp As VBComponent, ByRef CurProc As String, ByRef Iter As Long) As Long
On Error GoTo Err_LoadCodeStructProperty

  Dim ProcKinds(2) As Long              'Tableau contenant tous les types de prorits
  Dim i As Byte                         'Iterateur
  
  'Initialisation du tableau
  ProcKinds(0) = vbext_pk_Get   'Getter
  ProcKinds(1) = vbext_pk_Let   'Getter
  ProcKinds(2) = vbext_pk_Set   'Setter

  'Recherche du type de la proprit
  For i = 0 To 2                    'Une fois trouve les valeurs sont enregistres dans le tableau de structure, la dtction se fait sur l'erreur
    CodeStruct(Iter) = Array(Comp.Name, CurProc, Comp.CodeModule.ProcStartLine(CurProc, ProcKinds(i)), Comp.CodeModule.ProcCountLines(CurProc, ProcKinds(i)), ProcKinds(i))
    Iter = Iter + 1                 'Incrmentation de l'itrateur
    ReDim Preserve CodeStruct(Iter) 'Reallocation dynamique du tableau de la structure du code
  Next
  
  LoadCodeStructProperty = Iter
  Exit Function

Err_LoadCodeStructProperty:     'Gestion d'erreur
  Iter = Iter - 1               'Si une erreur est produite ce n'est pas le bon type alors on dcremente  l'itrateur
  Resume Next                   'et on retourne dans la boucle.
End Function


'********************************************************
'*      SearchStartProcedure                            *
'********************************************************
'* Description:                                         *
'*      Recherche la premire procedure dans la table   *
'*    en mmoire dans le but de savoir par ou commencer.*
'*                                                      *
'* Parameters:                                          *
'*      SearchProc (String) : Procedure  chercher.     *
'*                                                      *
'********************************************************

Public Function SearchStartProcedure(ByRef SearchProc As String) As Long

  Dim Iter As Long                            'Iterateur
  
  For Iter = 1 To UBound(CodeStruct())        'Recherche dans le tableau de structure de code la methode de dpart
    If CodeStruct(Iter)(1) = SearchProc Then  'Si elle est trouve
      SearchStartProcedure = Iter             'L'iterateur est sauvegard (Il est utilis ensuite comme pointeur)
      Exit For                                'On casse la boucle (itrations suivantes inutiles)
    End If
  Next
  
End Function

'********************************************************
'*      ReadCallStack                                   *
'********************************************************
'* Description:                                         *
'*      Function recursive qui parcours les appels.     *
'*                                                      *
'* Parameters:                                          *
'*      SrchItemIter (Long) : Iterator qui defini l'item*
'*   recherch.                                         *
'*      Deep (Long - Optional) : La profondeur de       *
'*   l'appel.                                           *
'*                                                      *
'********************************************************

Public Sub ReadCallStack(ByRef SrchItemIter As Long, Optional ByRef Deep As Long = 0)
  Dim Iter As Long                      'Iterateur
  Dim i As Long                         'Iterateur
  Dim StrDeclaration As String          'Ligne contenant la dclaration de la methode
  Dim StrProc As String                 'Nom de la procdure
  Dim CM As CodeModule                  'Module de code dans lequel on cherche des informations
  
  'On cherche  les appels l'interieur de la procedure pointe par le parametre SrchItemIter.
  For i = CLng(CodeStruct(SrchItemIter)(2)) + 1 To CLng(CodeStruct(SrchItemIter)(2)) + CLng(CodeStruct(SrchItemIter)(3)) - 2
    For Iter = 0 To UBound(CodeStruct())                                                            'Parcours du tableau de structure
      If CodeStruct(Iter)(1) <> CodeStruct(SrchItemIter)(1) Then                                    'Si le nom de la methode dans laquelle on cherche est diffrente de celle actuellement recherche (par l'iteration du tableau de structure)
        With Application.VBE.ActiveVBProject.VBComponents(CodeStruct(SrchItemIter)(0)).CodeModule   'On utilise le module dans lequel on recherche les appels (pointeur pass en paramtre).
            If .Find(CodeStruct(Iter)(1), i, 1, i, 255, True, True) Then                            'On cherche par rapport au tableau ou l'on listes les mthodes si la mthode pointe se trouve sur la ligne
              analyseLine Trim(.Lines(i, 1)), SrchItemIter, i, Deep 'Analyse la ligne de code
              ReadCallStack Iter, Deep + 1   'Appel rcursif (pour analyser le contenu de l'appel detect
              i = i + 1                      'On passe a la ligne suivante
            End If
        End With
      End If
    Next
  Next
  
End Sub

'********************************************************
'*      ReadProcType                                    *
'********************************************************
'* Description:                                         *
'*      Determine le type de la proprite/procedure.   *
'*                                                      *
'* Parameters:                                          *
'*      Iter (Long) : Iterateur pour trouver le type    *
'*                                                      *
'* Retours:                                             *
'*      (String) : Le type de la procedure/proprite   *
'********************************************************

Public Function ReadProcType(ByRef Iter As Long) As String

  Select Case CodeStruct(Iter)(4)  'On pointe La valeur de l'iterateur
    Case vbext_pk_Proc
      ReadProcType = "Proc"        'Si c'est une procdure on retourne "Proc"
    Case vbext_pk_Get
      ReadProcType = "Get"         'Si c'est un getter on retourne "Get"
    Case vbext_pk_Let
      ReadProcType = "Let"         'Si c'est un getter Let on retourne "Let"
    Case vbext_pk_Set
      ReadProcType = "Set"         'Si c'est un setter on retourne "Set"
  End Select

End Function


'********************************************************
'*      SortStrArray                                    *
'********************************************************
'* Description:                                         *
'*      Tri un tableau de chaine de caractres dans le  *
'*   but de faire plus tard une recherche dichotomique. *
'*   (Pour une nouvelle version de cet outil).          *
'*      Pour l'instant, cela tri sur seulement sur le   *
'*   premier caratre.                                  *
'*                                                      *
'* Parameters:                                          *
'*      Arr() (Variant) : Tableau  trier.              *
'*                                                      *
'********************************************************

'Attention tableau a 2 dimentions
'WARNING - it is for table with 2 dimentions

Public Sub SortStrArray(ByRef Arr() As Variant)
  Dim LenArray As Long                  'Longueur du tableau
  Dim Iterator As Long                  'Iterateur
  Dim SaveIterator As Long              'Sauvegarde de l'iterateur
  Dim StockValue As Variant             'Stockage d'une valeur / d'un objet (pour faire l'change)
  
  LenArray = UBound(Arr())              'Calcul de la longueur du tableau
  
  If LenArray >= 2 Then                 'Si le tableau possede plus de 1 lment (si il n'y a qu'un lment ca ne sert a rien de le trier)
    Iterator = 1                        'Initialisation de l'iterateur (1 car il y a au moins 1 lment dans le tableau)
    While Iterator <= LenArray          'Tant que l'iterateur ne dpasse pas la capacit du tableau.
      While Asc(Left(Arr(Iterator - 1)(1), 1)) > Asc(Left(Arr(Iterator)(1), 1)) 'Tant que la valeur / objet positionne avant l'itrateur est plus grand que la valeur / objet l ou pointe l'iterateur
        'Dplacement des valeurs / objets
        StockValue = Arr(Iterator - 1)                'Stockage de la valeur / objet
        Arr(Iterator - 1) = Arr(Iterator)             'Dcallage de la valeur / objet
        Arr(Iterator) = StockValue                    'Ecriture de la valeur / objet stocke au nouvel emplacement
        If Iterator > 1 Then Iterator = Iterator - 1  'Reprise du tri sur la premiere position change
      Wend
      Iterator = Iterator + 1   'Incrementation de l'iterateur
    Wend
  End If
End Sub

'********************************************************
'*      DisplayMessage                                  *
'********************************************************
'* Description:                                         *
'*      Affiche un message de la methode dtecte.      *
'*                                                      *
'* Parameters:                                          *
'*      StrDeclare(String): Declaration de la methode.  *
'*      Iterator (Long): Iterateur.                     *
'*      LineNumber(Long): Numro de la ligne detectee.  *
'*      Deep(Long): Profondeur pour l'affichage.        *
'*      SrchItemIter(Long): Iterateur de l'item         *
'*            recherche.                                *
'*                                                      *
'********************************************************

Private Sub DisplayMessage(ByRef StrDeclare As String, ByRef Iterator As Long, ByRef LineNumber As Long, ByRef Deep As Long, ByRef SrchItemIter As Long)
        Dim StrProc As String
        
        'Traitement de chaine sur la ligne d'appel sauvegarde (pour afficher un message)
        StrProc = Right(StrDeclare, Len(StrDeclare) - InStr(StrDeclare, CodeStruct(Iterator)(1)) + 1)
        
        'Traitement de chaine pour affichage
        If InStrRev(StrProc, ")") > 0 Then StrProc = Left(StrProc, InStrRev(StrProc, ")"))
        
        'Affichage de l'appel dtct dans la fenetre de dbuggage.
        Debug.Print Space(2 * Deep) & CStr(CodeStruct(SrchItemIter)(1)) & " [" & CodeStruct(SrchItemIter)(0) & "(" & LineNumber & ") -> " & CodeStruct(Iterator)(0) & "." & CodeStruct(Iterator)(1) & "{" & ReadProcType(Iterator) & "}] - " & StrProc
        
End Sub


'********************************************************
'*      analyseLine                                     *
'********************************************************
'* Description:                                         *
'*      Analyse une ligne de code provenant d'un module.*
'*                                                      *
'* Parameters:                                          *
'*      StrIn(String): Ligne  analyser.                *
'*      Iterator (Long): Iterateur.                     *
'*      LineNumber(Long): Numero de la ligne            *
'*      Deep(Long): Profondeur pour l'affichage.        *
'*                                                      *
'********************************************************

Private Sub analyseLine(ByRef StrIn As String, ByRef Iterator As Long, ByRef LineNumber As Long, ByRef Deep As Long)
        Dim ParIndexS As Integer  'Index de dbut de bloc de sparateur parenthses
        Dim ParIndexE As Integer  'Index de fin de bloc de sparateur parenthses
        Dim SpaIndex As Integer   'Index de sparateur espace
        Dim i As Long             'Iterateur
        Dim StrWord As String     'Stockage d'un mot


        If Len(StrIn) > 0 Then                               'Si la ligne contiens des caractres
          If Left(StrIn, 1) = "'" Then Exit Sub              'Si il s'agit d'un commentaire
          ParIndexS = InStr(StrIn, "(")                      'Affectation dbut premier bloc de parenthses
          SpaIndex = InStr(StrIn, Space(1))                  'Affectation index permier espace dtect dans la chane
          
          If SpaIndex = 0 Then SpaIndex = 32767              'Si l'index est  0, le caractre espace n'est pas prsent. On force l'index pour garder la cohrence du programme
          If ParIndexS = 0 Then ParIndexS = 32767            'Si l'index est  0, le caractre parenthese n'est pas prsent. On force l'index pour garder la cohrence du programme
          
          If SpaIndex = 32767 And ParIndexS = 32767 Then
          'Si les index de debut de paranethese et et d'espace sont a 0 c'est qu'il n'existe plus de dlimteurs sur la ligne
          '  Debug.Print "Mot " & StrIn: Stop
            For i = 0 To UBound(CodeStruct())               'Parcours du tableau de structure
              If CodeStruct(i)(1) = StrIn Then              'Si le mot est trouv dans le tableau de structure
                DisplayMessage StrIn, i, LineNumber, Deep, Iterator   'Affichage de l'appel detect
                Exit For                                    'Inutile de parcourir le reste du tableau
              End If
            Next
            Exit Sub
          End If

          If SpaIndex < ParIndexS Then                      'Si un espace se trouve avant une paranthese
            StrWord = Trim(Left(StrIn, SpaIndex))           'Isolation du mot
            'Debug.Print "Mot: " & StrWord & vbCrLf & "Dans la chaine: " & StrIn: Stop
            analyseLine Trim(Right(StrIn, Len(StrIn) - SpaIndex)), Iterator, LineNumber, Deep  'Analyse de tout ce qui se trouve apres le mot (reste de la chaine)
            
            For i = 0 To UBound(CodeStruct())               'Parcours du tableau de structure
              If CodeStruct(i)(1) = StrWord Then            'Si le mot est trouv dans le tableau de structure
                DisplayMessage StrIn, i, LineNumber, Deep, Iterator   'Affichage de l'appel detect
                Exit For                                    'Inutile de parcourir le reste du tableau
              End If
            Next
          Else                                              'Si une paranthese se trouve avant un espace
            'Gestion des parentheses
            Dim stockStrCur As Long                         'Index curseur
            Dim stockStrStart As Long                       'Index parenthse ouvrante
            Dim stockStrEnd As Long                         'Index parenthse fermante au bloc observ
            Dim cpt As Integer                              'Compteur
            
            stockStrCur = 0                                 'Initialisations
            stockStrEnd = 0
            cpt = 1
            
            'Debug.Print "Parenthese " & StrIn: Stop
            
            stockStrStart = InStr(stockStrCur + 1, StrIn, "(") 'detection parenthese ouvrante
                  
            'dtecte le nombre de nouveaux blocs jusqu'a la premire parenthese fermante
            Dim c As String * 1       'Caractere de chaine a analyser
            Dim chaine As String      'Resultat des operations de chaine
            Dim Detected As Boolean   'Pour detecter la fin de bloc
            
            'Recupere la partie droite de la chaine pour isoler le bloc de parentheses dtectes
            chaine = Right(StrIn, Len(StrIn) - InStr(StrIn, "("))
                                  
            For i = 1 To Len(chaine)  'Parcours de la chaine
              c = Mid(chaine, i, 1)   'lecture du caractere
              Select Case c           'Analyse du caractre
                Case "(":             'Si nouveau bloc suplmentaire incrmentation du compteur de blocs (cas ou il y a un bloc dans un bloc => (xxx (yyy, nnn)) par exemple)
                  cpt = cpt + 1     'Incrementation compteur
                Case ")":             'Si une fin de bloc est dtecte dcrmentation du compteur
                  cpt = cpt - 1     'Decrementation compteur
              End Select
              If cpt = 0 Then       'Si le compteur est a 0 la fin du bloc que l'on souhaite isol est dtect (cas ou il y aurai plusieurs bloc successifs => (xxx), (yyy) par exemple)
                Detected = True     'Mettre le dtection de fin de bloc a vrai
                Exit For            'Casser la bloucle => inutile d'analyser la suite de la ligne
              End If
            Next i
                                  
            
            
            If Detected Then analyseLine Left(chaine, i - 1), Iterator, LineNumber, Deep 'si la fin de bloc est dtecte, dclenchement de l'analyse de son contenu
            StrWord = Trim(Left(StrIn, stockStrStart - 1))  'isolation du mot qui a appel le bloc de parentheses
            'Debug.Print "Mot par:" & StrWord

            For i = 0 To UBound(CodeStruct())             'Parcours du tableau de structure
              If CodeStruct(i)(1) = StrWord Then          'Si le mot est trouve comme methode
                DisplayMessage StrIn, i, LineNumber, Deep, Iterator 'on affiche le message d'appel
                Exit For                                  'on casse la boucle
              End If
            Next
          End If
        End If
End Sub
